@@ -5,9 +5,10 @@
  */
 
 #include "config.h"
-#include "adw-action-row.h"
+#include "adw-action-row-private.h"
 
-#include "adw-macros-private.h"
+#include "adw-marshalers.h"
+#include "adw-widget-utils-private.h"
 
 /**
  * AdwActionRow:
@@ -46,6 +47,11 @@
  *
  * It contains subnodes `label.title` and `label.subtitle` representing
  * respectively the title label and subtitle label.
+ *
+ * `AdwActionRow` can use the
+ * [`.property`](style-classes.html#property-rows) style class to emphasize
+ * the row subtitle instead of the row title, which is useful for
+ * displaying read-only properties.
  */
 
 typedef struct
@@ -103,6 +109,47 @@ string_is_not_empty (AdwActionRow *self,
   return string && string[0];
 }
 
+static void
+pressed_cb (AdwActionRow *self,
+            int           n_press,
+            double        x,
+            double        y,
+            GtkGesture   *gesture)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+  GtkWidget *picked;
+  GtkEditable *delegate;
+
+  picked = gtk_widget_pick (GTK_WIDGET (self), x, y, GTK_PICK_DEFAULT);
+
+  if (picked != GTK_WIDGET (self) &&
+      picked != priv->header &&
+      picked != GTK_WIDGET (priv->prefixes) &&
+      picked != GTK_WIDGET (priv->suffixes)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+
+    return;
+  }
+
+  if (!GTK_IS_EDITABLE (self)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  delegate = gtk_editable_get_delegate (GTK_EDITABLE (self));
+
+  while (delegate && !GTK_IS_TEXT (delegate))
+    delegate = gtk_editable_get_delegate (delegate);
+
+  if (!delegate || !GTK_IS_TEXT (delegate)) {
+    gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_DENIED);
+    return;
+  }
+
+  gtk_text_grab_focus_without_selecting (GTK_TEXT (delegate));
+  gtk_gesture_set_state (gesture, GTK_EVENT_SEQUENCE_CLAIMED);
+}
+
 static void
 row_activated_cb (AdwActionRow  *self,
                   GtkListBoxRow *row)
@@ -234,7 +281,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   klass->activate = adw_action_row_activate_real;
 
   /**
-   * AdwActionRow:subtitle: (attributes org.gtk.Property.get=adw_action_row_get_subtitle org.gtk.Property.set=adw_action_row_set_subtitle)
+   * AdwActionRow:subtitle:
    *
    * The subtitle for this row.
    *
@@ -247,7 +294,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:icon-name: (attributes org.gtk.Property.get=adw_action_row_get_icon_name org.gtk.Property.set=adw_action_row_set_icon_name)
+   * AdwActionRow:icon-name:
    *
    * The icon name for this row.
    *
@@ -256,15 +303,15 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   props[PROP_ICON_NAME] =
     g_param_spec_string ("icon-name", NULL, NULL,
                          "",
-                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_DEPRECATED);
 
   /**
-   * AdwActionRow:activatable-widget: (attributes org.gtk.Property.get=adw_action_row_get_activatable_widget org.gtk.Property.set=adw_action_row_set_activatable_widget)
+   * AdwActionRow:activatable-widget:
    *
    * The widget to activate when the row is activated.
    *
    * The row can be activated either by clicking on it, calling
-   * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+   * [method@ActionRow.activate], or via mnemonics in the title.
    * See the [property@PreferencesRow:use-underline] property to enable
    * mnemonics.
    *
@@ -277,7 +324,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:title-lines: (attributes org.gtk.Property.get=adw_action_row_get_title_lines org.gtk.Property.set=adw_action_row_set_title_lines)
+   * AdwActionRow:title-lines:
    *
    * The number of lines at the end of which the title label will be ellipsized.
    *
@@ -290,7 +337,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:subtitle-lines: (attributes org.gtk.Property.get=adw_action_row_get_subtitle_lines org.gtk.Property.set=adw_action_row_set_subtitle_lines)
+   * AdwActionRow:subtitle-lines:
    *
    * The number of lines at the end of which the subtitle label will be
    * ellipsized.
@@ -304,7 +351,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwActionRow:subtitle-selectable: (attributes org.gtk.Property.get=adw_action_row_get_subtitle_selectable org.gtk.Property.set=adw_action_row_set_subtitle_selectable)
+   * AdwActionRow:subtitle-selectable:
    *
    * Whether the user can copy the subtitle from the label.
    *
@@ -329,9 +376,13 @@ adw_action_row_class_init (AdwActionRowClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   0,
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  adw_marshal_VOID__VOID,
                   G_TYPE_NONE,
                   0);
+  g_signal_set_va_marshaller (signals[SIGNAL_ACTIVATED],
+                              G_TYPE_FROM_CLASS (klass),
+                              adw_marshal_VOID__VOIDv);
 
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-action-row.ui");
@@ -343,6 +394,7 @@ adw_action_row_class_init (AdwActionRowClass *klass)
   gtk_widget_class_bind_template_child_private (widget_class, AdwActionRow, title);
   gtk_widget_class_bind_template_child_private (widget_class, AdwActionRow, title_box);
   gtk_widget_class_bind_template_callback (widget_class, string_is_not_empty);
+  gtk_widget_class_bind_template_callback (widget_class, pressed_cb);
 }
 
 static void
@@ -409,6 +461,7 @@ adw_action_row_add_prefix (AdwActionRow *self,
 
   g_return_if_fail (ADW_IS_ACTION_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_action_row_get_instance_private (self);
 
@@ -431,6 +484,7 @@ adw_action_row_add_suffix (AdwActionRow *self,
 
   g_return_if_fail (ADW_IS_ACTION_ROW (self));
   g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (gtk_widget_get_parent (widget) == NULL);
 
   priv = adw_action_row_get_instance_private (self);
 
@@ -469,7 +523,7 @@ adw_action_row_remove (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle: (attributes org.gtk.Method.get_property=subtitle)
+ * adw_action_row_get_subtitle:
  * @self: an action row
  *
  * Gets the subtitle for @self.
@@ -489,7 +543,7 @@ adw_action_row_get_subtitle (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle: (attributes org.gtk.Method.set_property=subtitle)
+ * adw_action_row_set_subtitle:
  * @self: an action row
  * @subtitle: the subtitle
  *
@@ -517,7 +571,7 @@ adw_action_row_set_subtitle (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * adw_action_row_get_icon_name:
  * @self: an action row
  *
  * Gets the icon name for @self.
@@ -539,7 +593,7 @@ adw_action_row_get_icon_name (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * adw_action_row_set_icon_name:
  * @self: an action row
  * @icon_name: (nullable): the icon name
  *
@@ -568,7 +622,7 @@ adw_action_row_set_icon_name (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_activatable_widget: (attributes org.gtk.Method.get_property=activatable-widget)
+ * adw_action_row_get_activatable_widget:
  * @self: an action row
  *
  * Gets the widget activated when @self is activated.
@@ -601,14 +655,14 @@ activatable_widget_weak_notify (gpointer  data,
 }
 
 /**
- * adw_action_row_set_activatable_widget: (attributes org.gtk.Method.set_property=activatable-widget)
+ * adw_action_row_set_activatable_widget:
  * @self: an action row
  * @widget: (nullable): the target widget
  *
  * Sets the widget to activate when @self is activated.
  *
  * The row can be activated either by clicking on it, calling
- * [method@ActionRow.activate], or via mnemonics in the title or the subtitle.
+ * [method@ActionRow.activate], or via mnemonics in the title.
  * See the [property@PreferencesRow:use-underline] property to enable mnemonics.
  *
  * The target widget will be activated by emitting the
@@ -663,7 +717,7 @@ adw_action_row_set_activatable_widget (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_title_lines: (attributes org.gtk.Method.get_property=title-lines)
+ * adw_action_row_get_title_lines:
  * @self: an action row
  *
  * Gets the number of lines at the end of which the title label will be
@@ -685,7 +739,7 @@ adw_action_row_get_title_lines (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_title_lines: (attributes org.gtk.Method.set_property=title-lines)
+ * adw_action_row_set_title_lines:
  * @self: an action row
  * @title_lines: the number of lines at the end of which the title label will be ellipsized
  *
@@ -717,7 +771,7 @@ adw_action_row_set_title_lines (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle_lines: (attributes org.gtk.Method.get_property=subtitle-lines)
+ * adw_action_row_get_subtitle_lines:
  * @self: an action row
  *
  * Gets the number of lines at the end of which the subtitle label will be
@@ -739,7 +793,7 @@ adw_action_row_get_subtitle_lines (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle_lines: (attributes org.gtk.Method.set_property=subtitle-lines)
+ * adw_action_row_set_subtitle_lines:
  * @self: an action row
  * @subtitle_lines: the number of lines at the end of which the subtitle label will be ellipsized
  *
@@ -771,8 +825,8 @@ adw_action_row_set_subtitle_lines (AdwActionRow *self,
 }
 
 /**
- * adw_action_row_get_subtitle_selectable: (attributes org.gtk.Method.get_property=subtitle-selectable)
- * @self: a `AdwActionRow`
+ * adw_action_row_get_subtitle_selectable:
+ * @self: an action row
  *
  * Gets whether the user can copy the subtitle from the label
  *
@@ -791,8 +845,8 @@ adw_action_row_get_subtitle_selectable (AdwActionRow *self)
 }
 
 /**
- * adw_action_row_set_subtitle_selectable: (attributes org.gtk.Method.set_property=subtitle-selectable)
- * @self: a `AdwActionRow`
+ * adw_action_row_set_subtitle_selectable:
+ * @self: an action row
  * @subtitle_selectable: `TRUE` if the user can copy the subtitle from the label
  *
  * Sets whether the user can copy the subtitle from the label
@@ -832,3 +886,24 @@ adw_action_row_activate (AdwActionRow *self)
 
   ADW_ACTION_ROW_GET_CLASS (self)->activate (self);
 }
+
+void
+adw_action_row_set_expand_suffixes (AdwActionRow *self,
+                                    gboolean      expand)
+{
+  AdwActionRowPrivate *priv = adw_action_row_get_instance_private (self);
+
+  g_return_if_fail (ADW_IS_ACTION_ROW (self));
+
+  expand = !!expand;
+
+  if (expand) {
+    gtk_widget_set_hexpand (GTK_WIDGET (priv->title_box), FALSE);
+    gtk_label_set_natural_wrap_mode (priv->title, GTK_NATURAL_WRAP_NONE);
+    gtk_label_set_natural_wrap_mode (priv->subtitle, GTK_NATURAL_WRAP_NONE);
+  } else {
+    gtk_widget_set_hexpand (GTK_WIDGET (priv->title_box), TRUE);
+    gtk_label_set_natural_wrap_mode (priv->title, GTK_NATURAL_WRAP_INHERIT);
+    gtk_label_set_natural_wrap_mode (priv->subtitle, GTK_NATURAL_WRAP_INHERIT);
+  }
+}
