@@ -9,6 +9,7 @@
 #include "config.h"
 #include "adw-message-dialog.h"
 
+#include "adw-bin.h"
 #include "adw-gizmo-private.h"
 #include "adw-gtkbuilder-utils-private.h"
 #include "adw-marshalers.h"
@@ -72,8 +73,7 @@
  * ## Async API
  *
  * `AdwMessageDialog` can also be used via the [method@MessageDialog.choose]
- * method. This API follows the GIO async pattern, and the result can be
- * obtained by calling [method@MessageDialog.choose_finish], for example:
+ * method. This API follows the GIO async pattern, for example:
  *
  * ```c
  * static void
@@ -149,27 +149,18 @@
  * `AdwMessageDialog` uses the `GTK_ACCESSIBLE_ROLE_DIALOG` role.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 
-/**
- * AdwResponseAppearance:
- * @ADW_RESPONSE_DEFAULT: the default appearance.
- * @ADW_RESPONSE_SUGGESTED: used to denote important responses such as the
- *     affirmative action.
- * @ADW_RESPONSE_DESTRUCTIVE: used to draw attention to the potentially damaging
- *     consequences of using the response. This appearance acts as a warning to
- *     the user.
- *
- * Describes the possible styles of [class@MessageDialog] response buttons.
- *
- * See [method@MessageDialog.set_response_appearance].
- *
- * Since: 1.2
- */
-
-#define DIALOG_MARGIN 30
-#define DIALOG_MAX_WIDTH 550
+#define DIALOG_MARGIN_VERT 20
+#define DIALOG_MARGIN_HORZ 30
+#define DIALOG_MAX_WIDTH 372 /* sp, not px */
+#define DIALOG_MAX_WIDE_WIDTH 600 /* sp, not px */
+#define DIALOG_PREFERRED_WIDTH 300 /* sp, not px */
 #define DIALOG_MIN_WIDTH 300
+#define BUTTON_SPACING 12
+#define RESPONSE_HORZ_PADDING 48
+#define RESPONSE_HORZ_PADDING_SHORT 36
 
 typedef struct {
   AdwMessageDialog *dialog;
@@ -179,13 +170,15 @@ typedef struct {
   gboolean enabled;
 
   GtkWidget *button;
-  GtkWidget *separator;
 } ResponseInfo;
 
 typedef struct
 {
+  GtkWidget *heading_bin;
   GtkWidget *heading_label;
+  GtkWidget *heading_label_small;
   GtkWidget *body_label;
+  GtkWidget *child_bin;
   GtkBox *message_area;
   GtkWidget *response_area;
 
@@ -265,6 +258,17 @@ parent_size_cb (AdwMessageDialog *self)
 
   priv->parent_width = w;
   priv->parent_height = h;
+
+  if (priv->parent_width < 450)
+    gtk_widget_add_css_class (GTK_WIDGET (self), "narrow");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
+
+  if (priv->parent_height < 360)
+    gtk_widget_add_css_class (GTK_WIDGET (self), "short");
+  else
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "short");
+
   gtk_widget_queue_resize (GTK_WIDGET (self));
 }
 
@@ -330,6 +334,9 @@ parent_unrealize_cb (AdwMessageDialog *self)
 
   priv->parent_width = -1;
   priv->parent_height = -1;
+
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "short");
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
 }
 
 static void
@@ -342,6 +349,9 @@ parent_window_notify_cb (AdwMessageDialog *self)
   priv->parent_window = NULL;
   priv->parent_width = -1;
   priv->parent_height = -1;
+
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "short");
+  gtk_widget_remove_css_class (GTK_WIDGET (self), "narrow");
 }
 
 static void
@@ -418,15 +428,14 @@ create_response_button (AdwMessageDialog *self,
 {
   GtkWidget *button = gtk_button_new_with_mnemonic (info->label);
 
-  gtk_widget_add_css_class (button, "flat");
   gtk_button_set_can_shrink (GTK_BUTTON (button), TRUE);
 
   switch (info->appearance) {
   case ADW_RESPONSE_SUGGESTED:
-    gtk_widget_add_css_class (button, "suggested");
+    gtk_widget_add_css_class (button, "suggested-action");
     break;
   case ADW_RESPONSE_DESTRUCTIVE:
-    gtk_widget_add_css_class (button, "destructive");
+    gtk_widget_add_css_class (button, "destructive-action");
     break;
   case ADW_RESPONSE_DEFAULT:
   default:
@@ -529,6 +538,105 @@ adw_message_dialog_map (GtkWidget *widget)
   }
 }
 
+static GtkSizeRequestMode
+get_heading_request_mode (GtkWidget *widget)
+{
+  return GTK_SIZE_REQUEST_HEIGHT_FOR_WIDTH;
+}
+
+static void
+measure_heading (GtkWidget      *widget,
+                 GtkOrientation  orientation,
+                 int             for_size,
+                 int            *minimum,
+                 int            *natural,
+                 int            *minimum_baseline,
+                 int            *natural_baseline)
+{
+  AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+  int large_min, large_nat, small_min, small_nat;
+
+  if (gtk_widget_has_css_class (GTK_WIDGET (self), "short")) {
+    gtk_widget_measure (priv->heading_label_small, orientation, for_size,
+                        minimum, natural, NULL, NULL);
+
+    if (minimum_baseline)
+      *minimum_baseline = -1;
+    if (natural_baseline)
+      *natural_baseline = -1;
+
+    return;
+  }
+
+  gtk_widget_measure (priv->heading_label, orientation, for_size,
+                      &large_min, &large_nat, NULL, NULL);
+  gtk_widget_measure (priv->heading_label_small, orientation, for_size,
+                      &small_min, &small_nat, NULL, NULL);
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL) {
+    if (minimum)
+      *minimum = MIN (large_min, small_min);
+    if (natural)
+      *natural = MAX (large_nat, small_nat);
+  } else {
+    if (for_size < 0) {
+      if (minimum)
+        *minimum = MAX (large_min, small_min);
+      if (natural)
+        *natural = MAX (large_nat, small_nat);
+    } else {
+      int large_width;
+
+      gtk_widget_measure (priv->heading_label, GTK_ORIENTATION_HORIZONTAL, -1,
+                          NULL, &large_width, NULL, NULL);
+
+      if (minimum)
+        *minimum = (large_width > for_size) ? small_min : large_min;
+      if (natural)
+        *natural = (large_width > for_size) ? small_nat : large_nat;
+    }
+  }
+
+  if (minimum_baseline)
+    *minimum_baseline = -1;
+  if (natural_baseline)
+    *natural_baseline = -1;
+}
+
+static void
+allocate_heading (GtkWidget *widget,
+                  int        width,
+                  int        height,
+                  int        baseline)
+{
+  AdwMessageDialog *self = ADW_MESSAGE_DIALOG (gtk_widget_get_root (widget));
+  AdwMessageDialogPrivate *priv = adw_message_dialog_get_instance_private (self);
+  gboolean small_label;
+
+  if (gtk_widget_has_css_class (GTK_WIDGET (self), "short")) {
+    small_label = TRUE;
+  } else {
+    int large_nat;
+
+    gtk_widget_measure (priv->heading_label, GTK_ORIENTATION_HORIZONTAL, -1,
+                        NULL, &large_nat, NULL, NULL);
+
+    small_label = large_nat > width;
+  }
+
+  if (gtk_widget_get_child_visible (priv->heading_label) == small_label)
+    gtk_widget_set_child_visible (priv->heading_label, !small_label);
+
+  if (gtk_widget_get_child_visible (priv->heading_label_small) != small_label)
+    gtk_widget_set_child_visible (priv->heading_label_small, small_label);
+
+  if (small_label)
+    gtk_widget_allocate (priv->heading_label_small, width, height, baseline, NULL);
+  else
+    gtk_widget_allocate (priv->heading_label, width, height, baseline, NULL);
+}
+
 static void
 measure_responses_do (AdwMessageDialog *self,
                       gboolean          compact,
@@ -562,17 +670,9 @@ measure_responses_do (AdwMessageDialog *self,
       nat += child_nat;
     }
 
-    if (response->separator) {
-      gtk_widget_measure (response->separator, orientation, -1,
-                          &child_min, &child_nat, NULL, NULL);
-
-    if (horiz == compact) {
-        min = MAX (min, child_min);
-        nat = MAX (nat, child_nat);
-      } else {
-        min += child_min;
-        nat += child_nat;
-      }
+    if (horiz != compact && l->next) {
+      min += BUTTON_SPACING;
+      nat += BUTTON_SPACING;
     }
   }
 
@@ -609,11 +709,14 @@ measure_responses (GtkWidget      *widget,
     measure_responses_do (self, FALSE, orientation, NULL, natural);
   } else {
     int wide_nat = 0;
+    gboolean use_compact_layout;
 
     if (for_size >= 0)
       measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
 
-    measure_responses_do (self, for_size >= 0 && for_size < wide_nat,
+    use_compact_layout = for_size >= 0 && wide_nat > for_size &&
+                         !gtk_widget_has_css_class (GTK_WIDGET (self), "short");
+    measure_responses_do (self, use_compact_layout,
                           orientation, minimum, natural);
   }
 
@@ -636,7 +739,7 @@ allocate_responses (GtkWidget *widget,
 
   measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
 
-  compact = wide_nat > width;
+  compact = wide_nat > width && !gtk_widget_has_css_class (GTK_WIDGET (self), "short");
 
   if (compact)
     gtk_widget_add_css_class (widget, "compact");
@@ -651,16 +754,6 @@ allocate_responses (GtkWidget *widget,
       ResponseInfo *response = l->data;
       int child_height;
 
-      if (response->separator) {
-        gtk_widget_measure (response->separator, GTK_ORIENTATION_VERTICAL, -1,
-                            &child_height, NULL, NULL, NULL);
-
-        pos -= child_height;
-
-        gtk_widget_allocate (response->separator, width, child_height, -1,
-                             gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (0, pos)));
-      }
-
       gtk_widget_measure (response->button, GTK_ORIENTATION_VERTICAL, -1,
                           &child_height, NULL, NULL, NULL);
 
@@ -668,48 +761,19 @@ allocate_responses (GtkWidget *widget,
 
       gtk_widget_allocate (response->button, width, child_height, -1,
                            gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (0, pos)));
+
+      pos -= BUTTON_SPACING;
     }
   } else {
     gboolean is_rtl = gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL;
     int pos = is_rtl ? width : 0;
     int n_buttons = g_list_length (priv->responses);
-    int total_width = width;
-    int button_width;
+    int total_width = width - BUTTON_SPACING * MAX (0, (n_buttons - 1));
+    int button_width = (int) ceil ((double) total_width / n_buttons);
     GList *l;
 
     for (l = priv->responses; l; l = l->next) {
       ResponseInfo *response = l->data;
-      int separator_width;
-
-      if (!response->separator)
-        continue;
-
-      gtk_widget_measure (response->separator, GTK_ORIENTATION_HORIZONTAL, -1,
-                          &separator_width, NULL, NULL, NULL);
-
-      total_width -= separator_width;
-    }
-
-    button_width = (int) ceil ((double) total_width / n_buttons);
-
-    for (l = priv->responses; l; l = l->next) {
-      ResponseInfo *response = l->data;
-
-      if (response->separator) {
-        int separator_width;
-
-        gtk_widget_measure (response->separator, GTK_ORIENTATION_HORIZONTAL, -1,
-                            &separator_width, NULL, NULL, NULL);
-
-        if (is_rtl)
-          pos -= separator_width;
-
-        gtk_widget_allocate (response->separator, separator_width, height, -1,
-                             gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (pos, 0)));
-
-        if (!is_rtl)
-          pos += separator_width;
-      }
 
       button_width = MIN (button_width, total_width);
 
@@ -722,7 +786,9 @@ allocate_responses (GtkWidget *widget,
                            gsk_transform_translate (NULL, &GRAPHENE_POINT_INIT (pos, 0)));
 
       if (!is_rtl)
-        pos += button_width;
+        pos += button_width + BUTTON_SPACING;
+      else
+        pos -= BUTTON_SPACING;
     }
   }
 }
@@ -748,26 +814,55 @@ adw_message_dialog_measure (GtkWidget      *widget,
                                                                NULL, NULL);
 
   if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-    int wide_nat, narrow_nat;
+    gboolean is_short = gtk_widget_has_css_class (GTK_WIDGET (self), "short");
+    int wide_nat, narrow_nat, heading_nat;
+    int max_width = adw_length_unit_to_px (ADW_LENGTH_UNIT_SP,
+                                           DIALOG_MAX_WIDTH,
+                                           gtk_widget_get_settings (widget));
+    int pref_width = adw_length_unit_to_px (ADW_LENGTH_UNIT_SP,
+                                            DIALOG_PREFERRED_WIDTH,
+                                            gtk_widget_get_settings (widget));
 
     min_size = MAX (min_size, DIALOG_MIN_WIDTH);
 
     if (priv->parent_window) {
-      max_size = priv->parent_width - DIALOG_MARGIN * 2;
-      max_size = MIN (max_size, DIALOG_MAX_WIDTH);
+      max_size = priv->parent_width - DIALOG_MARGIN_HORZ * 2;
+      max_size = MIN (max_size, max_width);
     } else {
-      max_size = DIALOG_MAX_WIDTH;
+      max_size = max_width;
+    }
+
+    if (gtk_widget_get_visible (priv->heading_bin)) {
+      gtk_widget_measure (priv->heading_bin, orientation, -1,
+                          NULL, &heading_nat, NULL, NULL);
+    } else {
+      heading_nat = 0;
     }
 
     measure_responses_do (self, FALSE, GTK_ORIENTATION_HORIZONTAL, NULL, &wide_nat);
     measure_responses_do (self, TRUE, GTK_ORIENTATION_HORIZONTAL, NULL, &narrow_nat);
 
-    narrow_nat = MAX (narrow_nat, DIALOG_MIN_WIDTH);
+    if (is_short) {
+      wide_nat += RESPONSE_HORZ_PADDING_SHORT;
+      narrow_nat += RESPONSE_HORZ_PADDING_SHORT;
+    } else {
+      wide_nat += RESPONSE_HORZ_PADDING;
+      narrow_nat += RESPONSE_HORZ_PADDING;
+    }
 
-    if (max_size < wide_nat)
+    narrow_nat = MAX (narrow_nat, pref_width);
+
+    if (is_short) {
+      max_size = adw_length_unit_to_px (ADW_LENGTH_UNIT_SP,
+                                        DIALOG_MAX_WIDE_WIDTH,
+                                        gtk_widget_get_settings (widget));
+
+      max_size = MIN (max_size, wide_nat);
+    } else if (wide_nat > min_size) {
       max_size = MIN (max_size, narrow_nat);
+    }
   } else {
-    max_size = priv->parent_height - DIALOG_MARGIN * 2;
+    max_size = priv->parent_height - DIALOG_MARGIN_VERT * 2;
   }
 
   max_size = MAX (min_size, max_size);
@@ -902,11 +997,12 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
   window_class->close_request = adw_message_dialog_close_request;
 
   /**
-   * AdwMessageDialog:heading: (attributes org.gtk.Property.get=adw_message_dialog_get_heading org.gtk.Property.set=adw_message_dialog_set_heading)
+   * AdwMessageDialog:heading:
    *
    * The heading of the dialog.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_HEADING] =
     g_param_spec_string ("heading", NULL, NULL,
@@ -914,13 +1010,14 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:heading-use-markup: (attributes org.gtk.Property.get=adw_message_dialog_get_heading_use_markup org.gtk.Property.set=adw_message_dialog_set_heading_use_markup)
+   * AdwMessageDialog:heading-use-markup:
    *
    * Whether the heading includes Pango markup.
    *
    * See [func@Pango.parse_markup].
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_HEADING_USE_MARKUP] =
     g_param_spec_boolean ("heading-use-markup", NULL, NULL,
@@ -928,11 +1025,12 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:body: (attributes org.gtk.Property.get=adw_message_dialog_get_body org.gtk.Property.set=adw_message_dialog_set_body)
+   * AdwMessageDialog:body:
    *
    * The body text of the dialog.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_BODY] =
     g_param_spec_string ("body", NULL, NULL,
@@ -940,13 +1038,14 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:body-use-markup: (attributes org.gtk.Property.get=adw_message_dialog_get_body_use_markup org.gtk.Property.set=adw_message_dialog_set_body_use_markup)
+   * AdwMessageDialog:body-use-markup:
    *
    * Whether the body text includes Pango markup.
    *
    * See [func@Pango.parse_markup].
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_BODY_USE_MARKUP] =
     g_param_spec_boolean ("body-use-markup", NULL, NULL,
@@ -954,13 +1053,14 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:extra-child: (attributes org.gtk.Property.get=adw_message_dialog_get_extra_child org.gtk.Property.set=adw_message_dialog_set_extra_child)
+   * AdwMessageDialog:extra-child:
    *
    * The child widget.
    *
    * Displayed below the heading and body.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_EXTRA_CHILD] =
     g_param_spec_object ("extra-child", NULL, NULL,
@@ -968,7 +1068,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:default-response: (attributes org.gtk.Property.get=adw_message_dialog_get_default_response org.gtk.Property.set=adw_message_dialog_set_default_response)
+   * AdwMessageDialog:default-response:
    *
    * The response ID of the default response.
    *
@@ -978,6 +1078,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    * will do nothing.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_DEFAULT_RESPONSE] =
     g_param_spec_string ("default-response", NULL, NULL,
@@ -985,7 +1086,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwMessageDialog:close-response: (attributes org.gtk.Property.get=adw_message_dialog_get_close_response org.gtk.Property.set=adw_message_dialog_set_close_response)
+   * AdwMessageDialog:close-response:
    *
    * The ID of the close response.
    *
@@ -997,6 +1098,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    * The default close response is `close`.
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   props[PROP_CLOSE_RESPONSE] =
     g_param_spec_string ("close-response", NULL, NULL,
@@ -1020,6 +1122,7 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
    * [property@MessageDialog:close-response].
    *
    * Since: 1.2
+   * Deprecated: 1.6: Use [class@AlertDialog].
    */
   signals[SIGNAL_RESPONSE] =
     g_signal_new ("response",
@@ -1038,8 +1141,11 @@ adw_message_dialog_class_init (AdwMessageDialogClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-message-dialog.ui");
 
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_bin);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_label);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, heading_label_small);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, body_label);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, child_bin);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, message_area);
   gtk_widget_class_bind_template_child_private (widget_class, AdwMessageDialog, response_area);
 
@@ -1069,11 +1175,21 @@ adw_message_dialog_init (AdwMessageDialog *self)
 
   gtk_widget_init_template (GTK_WIDGET (self));
 
+  gtk_widget_set_layout_manager (priv->heading_bin,
+                                 gtk_custom_layout_new (get_heading_request_mode,
+                                                        measure_heading,
+                                                        allocate_heading));
+
   gtk_widget_set_layout_manager (priv->response_area,
                                  gtk_custom_layout_new (get_responses_request_mode,
                                                         measure_responses,
                                                         allocate_responses));
 
+  adw_gizmo_set_focus_func (ADW_GIZMO (priv->heading_bin),
+                            (AdwGizmoFocusFunc) adw_widget_focus_child);
+  adw_gizmo_set_grab_focus_func (ADW_GIZMO (priv->heading_bin),
+                            (AdwGizmoGrabFocusFunc) adw_widget_grab_focus_child);
+
   adw_gizmo_set_focus_func (ADW_GIZMO (priv->response_area),
                             (AdwGizmoFocusFunc) adw_widget_focus_child);
   adw_gizmo_set_grab_focus_func (ADW_GIZMO (priv->response_area),
@@ -1337,6 +1453,7 @@ adw_message_dialog_buildable_init (GtkBuildableIface *iface)
  * Returns: the newly created `AdwMessageDialog`
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 GtkWidget *
 adw_message_dialog_new (GtkWindow  *parent,
@@ -1361,7 +1478,7 @@ adw_message_dialog_new (GtkWindow  *parent,
 }
 
 /**
- * adw_message_dialog_get_heading: (attributes org.gtk.Method.get_property=heading)
+ * adw_message_dialog_get_heading:
  * @self: a message dialog
  *
  * Gets the heading of @self.
@@ -1369,6 +1486,7 @@ adw_message_dialog_new (GtkWindow  *parent,
  * Returns: (nullable): the heading of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_heading (AdwMessageDialog *self)
@@ -1383,13 +1501,14 @@ adw_message_dialog_get_heading (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_heading: (attributes org.gtk.Method.set_property=heading)
+ * adw_message_dialog_set_heading:
  * @self: a message dialog
  * @heading: (nullable): the heading of @self
  *
  * Sets the heading of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_heading (AdwMessageDialog *self,
@@ -1406,7 +1525,9 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
     return;
 
   gtk_label_set_label (GTK_LABEL (priv->heading_label), heading);
-  gtk_widget_set_visible (priv->heading_label, heading && *heading);
+  gtk_label_set_label (GTK_LABEL (priv->heading_label_small), heading);
+
+  gtk_widget_set_visible (priv->heading_bin, heading && *heading);
 
   if (heading && *heading)
     gtk_widget_add_css_class (GTK_WIDGET (priv->message_area), "has-heading");
@@ -1419,7 +1540,7 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_heading_use_markup: (attributes org.gtk.Method.get_property=heading-use-markup)
+ * adw_message_dialog_get_heading_use_markup:
  * @self: a message dialog
  *
  * Gets whether the heading of @self includes Pango markup.
@@ -1427,6 +1548,7 @@ adw_message_dialog_set_heading (AdwMessageDialog *self,
  * Returns: whether @self uses markup for heading
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_get_heading_use_markup (AdwMessageDialog *self)
@@ -1441,7 +1563,7 @@ adw_message_dialog_get_heading_use_markup (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_heading_use_markup: (attributes org.gtk.Method.set_property=heading-use-markup)
+ * adw_message_dialog_set_heading_use_markup:
  * @self: a message dialog
  * @use_markup: whether to use markup for heading
  *
@@ -1450,6 +1572,7 @@ adw_message_dialog_get_heading_use_markup (AdwMessageDialog *self)
  * See [func@Pango.parse_markup].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_heading_use_markup (AdwMessageDialog *self,
@@ -1469,6 +1592,7 @@ adw_message_dialog_set_heading_use_markup (AdwMessageDialog *self,
   priv->heading_use_markup = use_markup;
 
   gtk_label_set_use_markup (GTK_LABEL (priv->heading_label), use_markup);
+  gtk_label_set_use_markup (GTK_LABEL (priv->heading_label_small), use_markup);
 
   update_window_title (self);
 
@@ -1487,6 +1611,7 @@ adw_message_dialog_set_heading_use_markup (AdwMessageDialog *self,
  * See [property@MessageDialog:heading].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_heading (AdwMessageDialog *self,
@@ -1536,6 +1661,7 @@ adw_message_dialog_format_heading (AdwMessageDialog *self,
  * See [property@MessageDialog:heading].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_heading_markup (AdwMessageDialog *self,
@@ -1569,7 +1695,7 @@ adw_message_dialog_format_heading_markup (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_body: (attributes org.gtk.Method.get_property=body)
+ * adw_message_dialog_get_body:
  * @self: a message dialog
  *
  * Gets the body text of @self.
@@ -1577,6 +1703,7 @@ adw_message_dialog_format_heading_markup (AdwMessageDialog *self,
  * Returns: the body of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_body (AdwMessageDialog *self)
@@ -1591,13 +1718,14 @@ adw_message_dialog_get_body (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_body: (attributes org.gtk.Method.set_property=body)
+ * adw_message_dialog_set_body:
  * @self: a message dialog
  * @body: the body of @self
  *
  * Sets the body text of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_body (AdwMessageDialog *self,
@@ -1625,7 +1753,7 @@ adw_message_dialog_set_body (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_body_use_markup: (attributes org.gtk.Method.get_property=body-use-markup)
+ * adw_message_dialog_get_body_use_markup:
  * @self: a message dialog
  *
  * Gets whether the body text of @self includes Pango markup.
@@ -1633,6 +1761,7 @@ adw_message_dialog_set_body (AdwMessageDialog *self,
  * Returns: whether @self uses markup for body text
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_get_body_use_markup (AdwMessageDialog *self)
@@ -1647,7 +1776,7 @@ adw_message_dialog_get_body_use_markup (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_body_use_markup: (attributes org.gtk.Method.set_property=body-use-markup)
+ * adw_message_dialog_set_body_use_markup:
  * @self: a message dialog
  * @use_markup: whether to use markup for body text
  *
@@ -1656,6 +1785,7 @@ adw_message_dialog_get_body_use_markup (AdwMessageDialog *self)
  * See [func@Pango.parse_markup].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_body_use_markup (AdwMessageDialog *self,
@@ -1690,6 +1820,7 @@ adw_message_dialog_set_body_use_markup (AdwMessageDialog *self,
  * See [property@MessageDialog:body].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_body (AdwMessageDialog *self,
@@ -1739,6 +1870,7 @@ adw_message_dialog_format_body (AdwMessageDialog *self,
  * See [property@MessageDialog:body].
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_format_body_markup (AdwMessageDialog *self,
@@ -1772,7 +1904,7 @@ adw_message_dialog_format_body_markup (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_extra_child: (attributes org.gtk.Method.get_property=extra-child)
+ * adw_message_dialog_get_extra_child:
  * @self: a message dialog
  *
  * Gets the child widget of @self.
@@ -1780,6 +1912,7 @@ adw_message_dialog_format_body_markup (AdwMessageDialog *self,
  * Returns: (nullable) (transfer none): the child widget of @self.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 GtkWidget *
 adw_message_dialog_get_extra_child (AdwMessageDialog *self)
@@ -1794,7 +1927,7 @@ adw_message_dialog_get_extra_child (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_extra_child: (attributes org.gtk.Method.set_property=extra-child)
+ * adw_message_dialog_set_extra_child:
  * @self: a message dialog
  * @child: (nullable): the child widget
  *
@@ -1803,6 +1936,7 @@ adw_message_dialog_get_extra_child (AdwMessageDialog *self)
  * The child widget is displayed below the heading and body.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_extra_child (AdwMessageDialog *self,
@@ -1821,13 +1955,9 @@ adw_message_dialog_set_extra_child (AdwMessageDialog *self,
   if (child == priv->child)
     return;
 
-  if (priv->child)
-    gtk_box_remove (priv->message_area, priv->child);
-
   priv->child = child;
-
-  if (priv->child)
-    gtk_box_append (priv->message_area, priv->child);
+  adw_bin_set_child (ADW_BIN (priv->child_bin), child);
+  gtk_widget_set_visible (priv->child_bin, child != NULL);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_EXTRA_CHILD]);
 }
@@ -1856,6 +1986,7 @@ adw_message_dialog_set_extra_child (AdwMessageDialog *self,
  * responses further.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_add_response (AdwMessageDialog *self,
@@ -1885,11 +2016,6 @@ adw_message_dialog_add_response (AdwMessageDialog *self,
   info->appearance = ADW_RESPONSE_DEFAULT;
   info->enabled = TRUE;
 
-  if (priv->responses) {
-    info->separator = gtk_separator_new (GTK_ORIENTATION_VERTICAL);
-    gtk_widget_set_parent (info->separator, priv->response_area);
-  }
-
   info->button = create_response_button (self, info);
   gtk_widget_set_parent (info->button, priv->response_area);
 
@@ -1923,6 +2049,7 @@ adw_message_dialog_add_response (AdwMessageDialog *self,
  * ```
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_add_responses (AdwMessageDialog *self,
@@ -1934,11 +2061,11 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
 
   g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
 
-  va_start (args, first_id);
-
   if (!first_id)
     return;
 
+  va_start (args, first_id);
+
   id = first_id;
   label = va_arg (args, const char *);
 
@@ -1955,6 +2082,47 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
   va_end (args);
 }
 
+/**
+ * adw_message_dialog_remove_response:
+ * @self: a message dialog
+ * @id: the response ID
+ *
+ * Removes a response from @self.
+ *
+ * Since: 1.5
+ * Deprecated: 1.6: Use [class@AlertDialog].
+ */
+void
+adw_message_dialog_remove_response (AdwMessageDialog *self,
+                                    const char       *id)
+{
+  AdwMessageDialogPrivate *priv;
+  ResponseInfo *info;
+
+  g_return_if_fail (ADW_IS_MESSAGE_DIALOG (self));
+  g_return_if_fail (id != NULL);
+
+  priv = adw_message_dialog_get_instance_private (self);
+  info = find_response (self, id);
+
+  if (!info) {
+    g_critical ("Trying to remove a response with id '%s' from an "
+                "AdwMessageDialog, but such a response does not exist",
+                id);
+    return;
+  }
+
+  if (priv->default_response == info->id)
+    gtk_window_set_default_widget (GTK_WINDOW (self), NULL);
+
+  gtk_widget_unparent (info->button);
+
+  priv->responses = g_list_remove (priv->responses, info);
+  g_hash_table_remove (priv->id_to_response, id);
+
+  response_info_free (info);
+}
+
 /**
  * adw_message_dialog_get_response_label:
  * @self: a message dialog
@@ -1967,6 +2135,7 @@ adw_message_dialog_add_responses (AdwMessageDialog *self,
  * Returns: the label of @response
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_response_label (AdwMessageDialog *self,
@@ -1995,6 +2164,7 @@ adw_message_dialog_get_response_label (AdwMessageDialog *self,
  * indicates a mnemonic.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_response_label (AdwMessageDialog *self,
@@ -2027,6 +2197,7 @@ adw_message_dialog_set_response_label (AdwMessageDialog *self,
  * Returns: the appearance of @response
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 AdwResponseAppearance
 adw_message_dialog_get_response_appearance (AdwMessageDialog *self,
@@ -2068,6 +2239,7 @@ adw_message_dialog_get_response_appearance (AdwMessageDialog *self,
  * Negative responses like Cancel or Close should use the default appearance.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
@@ -2090,14 +2262,14 @@ adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
   info->appearance = appearance;
 
   if (info->appearance == ADW_RESPONSE_SUGGESTED)
-    gtk_widget_add_css_class (info->button, "suggested");
+    gtk_widget_add_css_class (info->button, "suggested-action");
   else
-    gtk_widget_remove_css_class (info->button, "suggested");
+    gtk_widget_remove_css_class (info->button, "suggested-action");
 
   if (info->appearance == ADW_RESPONSE_DESTRUCTIVE)
-    gtk_widget_add_css_class (info->button, "destructive");
+    gtk_widget_add_css_class (info->button, "destructive-action");
   else
-    gtk_widget_remove_css_class (info->button, "destructive");
+    gtk_widget_remove_css_class (info->button, "destructive-action");
 }
 
 /**
@@ -2112,6 +2284,7 @@ adw_message_dialog_set_response_appearance (AdwMessageDialog      *self,
  * Returns: whether @response is enabled
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_get_response_enabled (AdwMessageDialog *self,
@@ -2146,6 +2319,7 @@ adw_message_dialog_get_response_enabled (AdwMessageDialog *self,
  * Responses are enabled by default.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
@@ -2171,7 +2345,7 @@ adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_default_response: (attributes org.gtk.Method.get_property=default-response)
+ * adw_message_dialog_get_default_response:
  * @self: a message dialog
  *
  * Gets the ID of the default response of @self.
@@ -2179,6 +2353,7 @@ adw_message_dialog_set_response_enabled (AdwMessageDialog *self,
  * Returns: (nullable): the default response ID
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_default_response (AdwMessageDialog *self)
@@ -2196,7 +2371,7 @@ adw_message_dialog_get_default_response (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_default_response: (attributes org.gtk.Method.set_property=default-response)
+ * adw_message_dialog_set_default_response:
  * @self: a message dialog
  * @response: (nullable): the default response ID
  *
@@ -2208,6 +2383,7 @@ adw_message_dialog_get_default_response (AdwMessageDialog *self)
  * will do nothing.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_default_response (AdwMessageDialog *self,
@@ -2236,7 +2412,7 @@ adw_message_dialog_set_default_response (AdwMessageDialog *self,
 }
 
 /**
- * adw_message_dialog_get_close_response: (attributes org.gtk.Method.get_property=close-response)
+ * adw_message_dialog_get_close_response:
  * @self: a message dialog
  *
  * Gets the ID of the close response of @self.
@@ -2244,6 +2420,7 @@ adw_message_dialog_set_default_response (AdwMessageDialog *self,
  * Returns: the close response ID
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_get_close_response (AdwMessageDialog *self)
@@ -2258,7 +2435,7 @@ adw_message_dialog_get_close_response (AdwMessageDialog *self)
 }
 
 /**
- * adw_message_dialog_set_close_response: (attributes org.gtk.Method.set_property=close-response)
+ * adw_message_dialog_set_close_response:
  * @self: a message dialog
  * @response: the close response ID
  *
@@ -2272,6 +2449,7 @@ adw_message_dialog_get_close_response (AdwMessageDialog *self)
  * The default close response is `close`.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_set_close_response (AdwMessageDialog *self,
@@ -2304,6 +2482,7 @@ adw_message_dialog_set_close_response (AdwMessageDialog *self,
  * Used to indicate that the user has responded to the dialog in some way.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_response (AdwMessageDialog *self,
@@ -2326,6 +2505,7 @@ adw_message_dialog_response (AdwMessageDialog *self,
  * Returns: whether @self has a response with the ID @response.
  *
  * Since: 1.2
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 gboolean
 adw_message_dialog_has_response (AdwMessageDialog *self,
@@ -2368,17 +2548,15 @@ choose_cancelled_cb (GCancellable *cancellable,
 
 /**
  * adw_message_dialog_choose:
- * @self: (transfer full): a message dialog
+ * @self: a message dialog
  * @cancellable: (nullable): a `GCancellable` to cancel the operation
  * @callback: (scope async): a callback to call when the operation is complete
- * @user_data: (closure callback): data to pass to @callback
+ * @user_data: data to pass to @callback
  *
  * This function shows @self to the user.
  *
- * The @callback will be called when the alert is dismissed. It should call
- * [method@MessageDialog.choose_finish] to obtain the result.
- *
  * Since: 1.3
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 void
 adw_message_dialog_choose (AdwMessageDialog    *self,
@@ -2412,6 +2590,7 @@ adw_message_dialog_choose (AdwMessageDialog    *self,
  *   [property@MessageDialog:close-response] if the call was cancelled.
  *
  * Since: 1.3
+ * Deprecated: 1.6: Use [class@AlertDialog].
  */
 const char *
 adw_message_dialog_choose_finish (AdwMessageDialog *self,
@@ -2426,3 +2605,4 @@ adw_message_dialog_choose_finish (AdwMessageDialog *self,
 
   return g_quark_to_string (id);
 }
+
